; GRAPHICS.ASM

public Blit       ; this makes the routine name callable from QuickBASIC
public PutBlock   ; this makes the routine name callable from QuickBASIC
public PutSprite  ; this makes the routine name callable from QuickBASIC

.model medium       ; this sets things up properly for use with QuickBASIC
.CODE

Blit    proc far
; Calling syntax: CALL Blit( x1%, y1%, x2%, y2%, w%, h% )
;   location on the stack:   16   14   12   10   8   6

    push bp             ; save old BP value
    mov bp,sp           ; BP now points to the parameters

    push ds
    push es
    push si
    push di
    pushf

blit_setup_pointers:
    mov bx, 0A000h      ; set up segments for MOVSB
    mov ds, bx          ; source segment
    mov es, bx          ; destination segment

    mov ax, [bp+14]     ; y1%
    mov bx, 320         ;       320
    mul bx              ;     *
    add ax, [bp+16]     ;           + x1%
    mov si, ax          ; si is source offset

    mov ax, [bp+10]     ; y2%
    mov bx, 320         ;       320
    mul bx              ;     *
    add ax, [bp+12]     ;           + x2%
    mov di, ax          ; di is destination offset

blit_setup_dimensions:
    mov bx, 320         ; horizontal jump between lines

blit_check_x_direction:
    mov ax, [bp+16]     ; x1% compared to
    cmp ax, [bp+12]     ; x2%
    jg blit_left_to_right

blit_right_to_left:
    std                 ; set reverse (right-to-left) operation
    add si, [bp+8]      ; adjust offsets
    add di, [bp+8]      ; adjust offsets
    dec si              ; adjust offsets
    dec di              ; adjust offsets

    add bx, [bp+8]      ; adjust interlinear offset
    jmp blit_check_y_direction

blit_left_to_right:
    cld                 ; set forward (left-to-right) operation
    sub bx, [bp+8]      ; adjust interlinear offset

blit_check_y_direction:
    mov ax, [bp+14]     ; y1% compared to
    cmp ax, [bp+10]     ; y2%
    jl blit_upwards

blit_downwards:

    xor ax, ax          ; set y counter to 0

blit_downwards_row:

    mov cx, [bp+8]      ; set x counter
    repnz movsb         ; transfer bytes

    add si, bx          ; add interlinear offset
    add di, bx          ; add interlinear offset

    inc ax              ; increment y counter
    cmp ax, [bp+6]      ; compare with height
    jl blit_downwards_row

    jmp blit_done

blit_upwards:

    mov ax, [bp+6]      ; (h
    dec ax              ;   -1)
    mov cx, 320         ;         320
    mul cx              ;       *
    add si, ax          ; compensate offset for upward
    add di, ax          ; movement

    xor ax, ax          ; set y counter to 0

    sub bx, [bp+8]
    sub bx, [bp+8]

blit_upwards_row:

    mov cx, [bp+8]      ; set x counter
    repnz movsb         ; transfer bytes

    sub si, bx          ; subtract interlinear offset
    sub di, bx          ; subtract interlinear offset

    inc ax              ; increment y counter
    cmp ax, [bp+6]      ; compare with height
    jl blit_upwards_row

    jmp blit_done

blit_done:

    popf
    pop di
    pop si
    pop es
    pop ds

    pop bp              ; restore old BP value
    ret 12              ; return (2 * # parameters)

Blit    endp

PutBlock    proc far
; Calling syntax: CALL PutBlock( x%, y%, w%, h%, bseg%, boff%, i% )
;   location on the stack:       18  16  14  12  10      8     6

    push bp             ; save old BP value
    mov bp,sp           ; BP now points to the parameters

    sub sp, 6

    push ds
    push es
    push si
    push di
    pushf

putblock_setup_pointers:
    mov bx, 0A000h      ; set up segments for MOVSB
    mov ds, [bp+10]     ; source segment
    mov es, bx          ; destination segment

    mov ax, [bp+6]      ; i%
    mov bx, 256         ;       256
    mul bx              ;     *
    add ax, [bp+8]      ;            + boff%
    mov si, ax          ; si is source offset

    mov ax, [bp+16]     ; y%
    mov bx, 320         ;       320
    mul bx              ;     *
    add ax, [bp+18]     ;           + x%
    mov di, ax          ; di is destination offset

putblock_setup_dimensions:
    mov bx, 320         ; horizontal jump between lines
    sub bx, [bp+14]     ;
    mov dx, [bp+14]
    mov [bp-02], dx     ; adjusted width
    mov word ptr [bp-04], 0      ; source increment
    mov dx, [bp+12]
    mov [bp-06], dx     ; adjusted height

putblock_clip_x_right:
    mov ax, [bp+18]     ; x%
    add ax, [bp+14]     ;    + w%
    sub ax, 320         ;         - 320
    jl putblock_clip_x_left
    add bx, ax          ; increase horizontal jump
    add [bp-4], ax      ; source increment
    sub [bp-2], ax      ; decrease width

putblock_clip_x_left:
    mov ax, 0           ; 0
    sub ax, [bp+18]     ;   - x%
    jl putblock_clip_y_top
    add si, ax          ; increase source offset
    add di, ax          ; increase dest offset
    add bx, ax          ; increase horizontal jump
    add [bp-4], ax      ; source increment
    sub [bp-2], ax      ; decrease width

putblock_clip_y_top:
    mov ax, 0           ; 0
    sub ax, [bp+16]     ;   - y%
    jl putblock_clip_y_bottom
    sub [bp-6], ax      ; decrease height
    mov dx, 16          ; make into lines
    mul dx              ;
    add si, ax          ; increase source offset
    mov dx, 20          ; make into lines
    mul dx              ;
    add di, ax          ; increase dest offset

putblock_clip_y_bottom:
    mov ax, [bp+16]     ; y%
    add ax, [bp+12]     ;    + h%
    sub ax, 200         ;         - 200
    jl putblock_copy_block
    sub [bp-6], ax      ; decrease height

putblock_copy_block:
    cld

    cmp word ptr [bp-2], 0  ;check clipped width > 0
    jle putblock_done
    cmp word ptr [bp-6], 0  ;check clipped height > 0
    jle putblock_done

    xor ax, ax          ; set y counter to 0

putblock_row:

    mov cx, [bp-2]      ; set x counter
    repnz movsb         ; transfer bytes

    add si, [bp-4]      ; source increment
    add di, bx          ; add interlinear offset

    inc ax              ; increment y counter
    cmp ax, [bp-6]      ; compare with height
    jl putblock_row

putblock_done:

    popf
    pop di
    pop si
    pop es
    pop ds

    add sp, 6

    pop bp              ; restore old BP value
    ret 14              ; return (2 * # parameters)

PutBlock    endp

PutSprite    proc far
; Calling syntax: CALL PutSprite( x%, y%, w%, h%, bseg%, boff%, i% )
;   location on the stack:        18  16  14  12  10      8     6

    push bp             ; save old BP value
    mov bp,sp           ; BP now points to the parameters

    sub sp, 6

    push ds
    push es
    push si
    push di
    pushf

putsprite_setup_pointers:
    mov bx, 0A000h      ; set up segments for MOVSB
    mov ds, [bp+10]     ; source segment
    mov es, bx          ; destination segment

    mov ax, [bp+6]      ; i%
    mov bx, 256         ;       256
    mul bx              ;     *
    add ax, [bp+8]      ;            + boff%
    mov si, ax          ; si is source offset

    mov ax, [bp+16]     ; y%
    mov bx, 320         ;       320
    mul bx              ;     *
    add ax, [bp+18]     ;           + x%
    mov di, ax          ; di is destination offset

putsprite_setup_dimensions:
    mov bx, 320         ; horizontal jump between lines
    sub bx, [bp+14]     ;
    mov dx, [bp+14]
    mov [bp-02], dx     ; adjusted width
    mov word ptr [bp-04], 0      ; source increment
    mov dx, [bp+12]
    mov [bp-06], dx     ; adjusted height

putsprite_clip_x_right:
    mov ax, [bp+18]     ; x%
    add ax, [bp+14]     ;    + w%
    sub ax, 320         ;         - 320
    jl putsprite_clip_x_left
    add bx, ax          ; increase horizontal jump
    add [bp-4], ax      ; source increment
    sub [bp-2], ax      ; decrease width

putsprite_clip_x_left:
    mov ax, 0           ; 0
    sub ax, [bp+18]     ;   - x%
    jl putsprite_clip_y_top
    add si, ax          ; increase source offset
    add di, ax          ; increase dest offset
    add bx, ax          ; increase horizontal jump
    add [bp-4], ax      ; source increment
    sub [bp-2], ax      ; decrease width

putsprite_clip_y_top:
    mov ax, 0           ; 0
    sub ax, [bp+16]     ;   - y%
    jl putsprite_clip_y_bottom
    sub [bp-6], ax      ; decrease height
    mov dx, 16          ; make into lines
    mul dx              ;
    add si, ax          ; increase source offset
    mov dx, 20          ; make into lines
    mul dx              ;
    add di, ax          ; increase dest offset

putsprite_clip_y_bottom:
    mov ax, [bp+16]     ; y%
    add ax, [bp+12]     ;    + h%
    sub ax, 200         ;         - 200
    jl putsprite_copy_block
    sub [bp-6], ax      ; decrease height

putsprite_copy_block:
    cld

    cmp word ptr [bp-2], 0  ;check clipped width > 0
    jle putsprite_done
    cmp word ptr [bp-6], 0  ;check clipped height > 0
    jle putsprite_done

    xor ax, ax          ; set y counter to 0

putsprite_row:

    mov cx, [bp-2]      ; set x counter

putsprite_row_loop:

    cmp byte ptr [ds:si], 255
    je putsprite_trans
    movsb               ; transfer bytes
    loopnz putsprite_row_loop
    jmp putsprite_row_done

putsprite_trans:
    inc si              ; skip pixel
    inc di              ; skip pixel
    loopnz putsprite_row_loop

putsprite_row_done:

    add si, [bp-4]      ; source increment
    add di, bx          ; add interlinear offset

    inc ax              ; increment y counter
    cmp ax, [bp-6]      ; compare with height
    jl putsprite_row

putsprite_done:

    popf
    pop di
    pop si
    pop es
    pop ds

    add sp, 6

    pop bp              ; restore old BP value
    ret 14              ; return (2 * # parameters)

PutSprite    endp

end
